'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.warn = warn;
exports.getPropertySize = getPropertySize;
exports.isMobileDevice = isMobileDevice;
exports.windowExists = windowExists;
exports.initWindowState = initWindowState;
exports.generateInternalId = generateInternalId;
exports.registerGlobalContextMenu = registerGlobalContextMenu;
exports.removeGlobalContextMenu = removeGlobalContextMenu;
exports.dispatchWindowEvent = dispatchWindowEvent;
exports.registerShowIntent = registerShowIntent;
exports.setLastTriggerData = setLastTriggerData;
exports.getLastTriggerData = getLastTriggerData;
exports.emitContextMenuEvent = emitContextMenuEvent;
exports.hideAllContextMenus = hideAllContextMenus;
exports.prepareContextMenuHandlers = prepareContextMenuHandlers;
exports.determineContextMenuPlacement = determineContextMenuPlacement;
/**
 * @author v1ndic4te
 * @copyright 2018
 * @licence GPL-3.0
 */

function warn() /*arguments*/{
    var args = Array.prototype.slice.call(arguments);
    args.unshift('[react-context-menu-wrapper]');
    console.warn.apply(this, args);
}

var ContextMenuEvent = exports.ContextMenuEvent = {
    Show: 'show',
    Hide: 'hide'
};

var InternalEvent = exports.InternalEvent = {
    TryShowContextMenu: '__rcmw-try-show',
    DoShowContextMenu: '__rcmw-do-show',
    HideAllContextMenus: '__rcmw-hide-all'
};

var TriggerType = exports.TriggerType = {
    Manual: 'manual', // When context menu is requested programmatically
    Native: 'native' // When context menu is requested via an input event
};

var TriggerContext = exports.TriggerContext = {
    Local: 'local', // When requested via a local listener
    Global: 'global', // When requested via a global listener
    Cancel: 'cancel' // When a cancel event is requested
};

function getPropertySize(node, property) {
    try {
        var value = window.getComputedStyle(node).getPropertyValue(property);
        return +value.replace(/[^\d.]+/g, '');
    } catch (error) {
        // Bad property, object, doesn't matter.
        return 0;
    }
}

var extractEventDetails = exports.extractEventDetails = function extractEventDetails(event) {
    var coordsObject = event;
    var isTouch = false;
    if (event.targetTouches) {
        coordsObject = event.targetTouches[0];
        isTouch = true;
    }
    return {
        preventDefault: event.cancelable ? function () {
            return event.preventDefault();
        } : null,
        triggerSource: event.currentTarget,
        triggerTarget: event.target,
        x: coordsObject.clientX,
        y: coordsObject.clientY,
        isTouch: isTouch
    };
};

function isMobileDevice() {
    return typeof window.orientation !== 'undefined' || navigator.userAgent.indexOf('IEMobile') !== -1;
}

function windowExists() {
    return typeof window !== 'undefined';
}

function initWindowState() {
    if (window._reactContextMenuWrapperData) return;

    window._reactContextMenuWrapper = {
        globalMenus: [], // Array of IDs of menus that are currently global
        lastTriggerDataMap: {}, // Map of event IDs to their last 'context-menu' events

        intentTimeout: null, // Holds the debounce timeout for context menu show intents
        intentWinner: null, // Holds the deepest menu intent at the moment debounce is triggered

        globalTouchTimeout: null, // Touch event timeout for global context menus
        touchTimeoutMap: {}, // Map of touch event timeouts for different IDs

        globalContextMenuEventListeners: [], // Array of global listeners defined by the user
        contextMenuEventListenersMap: {}, // Map of ID listeners defined by the user

        holdToShowInterval: 420 // Long press duration in milliseconds
    };

    var globalHandlers = prepareContextMenuHandlers();

    document.addEventListener('contextmenu', globalHandlers.onContextMenu);
    document.addEventListener('touchstart', globalHandlers.onTouchStart);
    document.addEventListener('touchmove', globalHandlers.onTouchEnd);
    document.addEventListener('touchend', globalHandlers.onTouchEnd);
    document.addEventListener('touchcancel', globalHandlers.onTouchEnd);
}

function generateInternalId() {
    return Math.random().toString(36).substring(6);
}

function registerGlobalContextMenu(internalId) {
    var menus = window._reactContextMenuWrapper.globalMenus;
    if (menus.length !== 0) {
        warn('You have registered multiple global context menus - menus will likely display incorrectly. It\'s ' + 'recommended you only have one global context menu.');
    }
    menus.push(internalId);
}

function removeGlobalContextMenu(internalId) {
    var menus = window._reactContextMenuWrapper.globalMenus;

    var index = menus.indexOf(internalId);
    if (index > -1) {
        menus.splice(index, 1);
    }
}

function dispatchWindowEvent(eventName) {
    var detail = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var event = void 0;
    if (typeof window.CustomEvent === 'function') {
        event = new window.CustomEvent(eventName, { detail: detail });
    } else {
        event = document.createEvent('CustomEvent');
        event.initCustomEvent(eventName, false, true, detail);
    }
    window.dispatchEvent(event);
}

var notifyIntentWinner = function notifyIntentWinner() {
    var store = window._reactContextMenuWrapper;
    var winner = store.intentWinner;
    store.intentTimeout = null;
    store.intentWinner = null;
    if (!winner) return;

    if (winner.eventDetails.triggerContext === TriggerContext.Cancel) {
        // Current show intent is a cancel command, do nothing.
        return;
    }

    dispatchWindowEvent(InternalEvent.DoShowContextMenu, winner);
};

/**
 * Called by 'ContextMenuWrapper' instances when they receive a show request. This method is responsible for resolving
 * which context menu should actually be shown.
 *
 * Chooses the context menu associated with the bottom-most DOM node in the tree.
 *
 * @param {object} data
 * @param {string} [data.internalId]
 * @param {string} [data.externalId]
 * @param {*} data.eventDetails       The object that triggered the context menu event.
 * @param {*} [data.data]             Data associated with the trigger.
 */
function registerShowIntent(data) {
    var store = window._reactContextMenuWrapper;

    // We know that whoever submitted the show intent is ready to show the context menu. Whether they win or not,
    // we can prevent the default behaviour of the event.
    if (data.eventDetails.preventDefault) data.eventDetails.preventDefault();

    if (store.intentWinner) {
        var ourDetails = data.eventDetails;
        var otherDetails = store.intentWinner.eventDetails;

        if (otherDetails.triggerContext === TriggerContext.Cancel) {
            // Do nothing because cancel commands always take precedence.
        } else if (ourDetails.triggerType === TriggerType.Manual) {
            // Do nothing because manual events always take precedence
            // (unless the previous event is a manual cancel command.
        } else if (ourDetails.triggerContext === TriggerContext.Global) {
            // Either the existing trigger is also global (which means it will be identical to us) or it is local,
            // in which case we would lose. Either way, we can just return.
            return;
        } else if (ourDetails.triggerContext === TriggerContext.Local && otherDetails.triggerContext === TriggerContext.Global) {
            // We won because we're local and the other intent is global.
        } else {
            var ourSource = ourDetails.triggerSource;
            var otherSource = otherDetails.triggerSource;

            if (!ourSource || !otherSource) {
                // If we got here, it means that we were triggered by a native event that didn't have proper targets
                // specified. The only thing we can do is just give up and assume the current intent lost.
                return;
            }

            // If we have higher precedence than our opponent, we win the intent.
            if (ourSource.contains(otherSource)) {
                return;
            }
        }

        // TODO: Handle the weird case when none of the sources contain the other.
    }

    store.intentWinner = data;
    clearTimeout(store.intentTimeout);
    store.intentTimeout = setTimeout(notifyIntentWinner, 5);
}

function setLastTriggerData(internalId, data) {
    var map = window._reactContextMenuWrapper.lastTriggerDataMap;
    map[internalId] = data;
}

function getLastTriggerData(internalId) {
    var map = window._reactContextMenuWrapper.lastTriggerDataMap;
    return map[internalId];
}

/**
 * @param {object} data
 * @param {string} [data.externalId]
 * @param {string} data.eventName
 * @param {*} data.data
 * @param {object} data.publicProps
 */
function emitContextMenuEvent(data) {
    var store = window._reactContextMenuWrapper;
    var id = data.externalId;

    var listenerArray = void 0;
    if (id) {
        var map = store.contextMenuEventListenersMap;
        if (!map[id]) map[id] = [];
        listenerArray = map[id];
    } else {
        listenerArray = store.globalContextMenuEventListeners;
    }

    for (var _iterator = listenerArray, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref = _iterator[_i++];
        } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref = _i.value;
        }

        var listener = _ref;

        listener(data.eventName, data.data, data.publicProps);
    }
}

/**
 * When both parameters are provided, registers a listener for a specific context menu ID. If the first argument is
 * null, registers a global context menu listener.
 *
 * @param {string} [id]
 * @param {function} listener
 */
var addContextMenuEventListener = exports.addContextMenuEventListener = function addContextMenuEventListener(id, listener) {
    if (!listener) warn('Tried to register a context menu state listener but no listener object was provided.');
    var store = window._reactContextMenuWrapper;

    var listenerArray = void 0;
    if (id) {
        var map = store.contextMenuEventListenersMap;
        if (!map[id]) map[id] = [];
        listenerArray = map[id];
    } else {
        listenerArray = store.globalContextMenuEventListeners;
    }

    if (listenerArray.indexOf(listener) !== -1) {
        warn('Tried to add the same event listener to \'' + id + '\' twice.');
    } else {
        listenerArray.push(listener);
    }
};

/**
 * When both parameters are provided, removes a listener from a specific context menu ID. If the first argument is
 * null, removes a global context menu listener.
 *
 * @param {string} [id]
 * @param {function} listener
 */
var removeContextMenuEventListener = exports.removeContextMenuEventListener = function removeContextMenuEventListener(id, listener) {
    if (!listener) warn('Tried to register a context menu state listener but no listener object was provided.');
    var store = window._reactContextMenuWrapper;

    var listenerArray = void 0;
    if (id) {
        var map = store.contextMenuEventListenersMap;
        if (!map[id]) map[id] = [];
        listenerArray = map[id];
    } else {
        listenerArray = store.globalContextMenuEventListeners;
    }

    var index = listenerArray.indexOf(listener);
    if (index > -1) {
        listenerArray.splice(index, 1);
    }
};

/**
 * @param {object} params
 * @param {string} [params.id]  External ID of the context menu
 * @param {*} [params.data]     Data associated with the event
 * @param {*} [params.event]    ContextMenu event that triggered the logic
 * @param {number} [params.x]   x-coordinate to show the menu at
 * @param {number} [params.y]   y-coordinate to show the menu at
 */
var showContextMenu = exports.showContextMenu = function showContextMenu() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var eventDetails = {
        triggerType: TriggerType.Manual,
        triggerContext: params.id ? TriggerContext.Local : TriggerContext.Global,
        x: 0,
        y: 0
    };
    if (params.event) {
        eventDetails = _extends({}, eventDetails, extractEventDetails(params.event));
    }
    if (params.x) eventDetails.x = params.x;
    if (params.y) eventDetails.y = params.y;
    dispatchWindowEvent(InternalEvent.TryShowContextMenu, {
        eventDetails: eventDetails,
        externalId: params.id,
        data: params.data
    });
};

function hideAllContextMenus() {
    dispatchWindowEvent(InternalEvent.HideAllContextMenus);
}

var cancelOtherContextMenus = exports.cancelOtherContextMenus = function cancelOtherContextMenus() {
    var eventDetails = {
        triggerType: TriggerType.Manual,
        triggerContext: TriggerContext.Cancel
    };
    registerShowIntent({ eventDetails: eventDetails });
};

/**
 * Prepares an object with handlers for different events.
 *
 * The long press logic comes from John Doherty's `library long-press`
 * @see https://github.com/john-doherty/long-press/blob/master/src/long-press.js
 *
 * @param {number} params
 * @param {string} [params.id]
 * @param {*} [params.data]
 */
function prepareContextMenuHandlers() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var store = window._reactContextMenuWrapper;
    var holdToShowInterval = store.holdToShowInterval;
    var id = params.id,
        data = params.data;

    var triggerContext = id ? TriggerContext.Local : TriggerContext.Global;

    var eventDetails = void 0;

    var prepareEventDetails = function prepareEventDetails(event) {
        return _extends({
            triggerType: TriggerType.Native,
            triggerContext: triggerContext
        }, extractEventDetails(event));
    };
    var dispatchShowRequest = function dispatchShowRequest(eventDetails) {
        dispatchWindowEvent(InternalEvent.TryShowContextMenu, { eventDetails: eventDetails, externalId: id, data: data });
    };

    var updateTimeout = function updateTimeout() {
        var newTimeout = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

        if (newTimeout) {
            // Persist new timeout
            if (id) store.touchTimeoutMap[id] = newTimeout;else store.globalTouchTimeout = newTimeout;
        } else {
            // Clear old timeout
            var oldTimeout = void 0;
            if (id) oldTimeout = store.touchTimeoutMap[id];else oldTimeout = store.globalTouchTimeout;
            clearTimeout(oldTimeout);
        }
    };

    return {
        onContextMenu: function onContextMenu(event) {
            var eventDetails = prepareEventDetails(event);
            dispatchShowRequest(eventDetails);
        },
        onTouchStart: function onTouchStart(event) {
            eventDetails = prepareEventDetails(event);
            var timeout = setTimeout(function () {
                dispatchWindowEvent(InternalEvent.TryShowContextMenu, { eventDetails: eventDetails, externalId: id, data: data });
            }, holdToShowInterval);
            updateTimeout(timeout);
        },
        onTouchMove: function onTouchMove() {
            return updateTimeout();
        },
        onTouchEnd: function onTouchEnd() {
            return updateTimeout();
        },
        onTouchCancel: function onTouchCancel() {
            return updateTimeout();
        }
    };
}

/**
 * @param {object} params
 * @param {number} [params.clickX]
 * @param {number} [params.clickY]
 * @param {*} [params.domNode]
 * @returns {{x: number, y: number}}
 */
function determineContextMenuPlacement(params) {
    var windowWidth = window.innerWidth;
    var windowHeight = window.innerHeight;

    // If no `clickX` and `clickY` were provided, assuming the click was in the middle of the screen.
    var clickX = params.clickX ? params.clickX : windowWidth / 2;
    var clickY = params.clickY ? params.clickY : windowHeight / 2;

    // Iterate through context menu children to figure out its actual height
    var menuWidth = void 0,
        menuHeight = void 0;
    var sizerNode = params.domNode;
    if (sizerNode) {
        do {
            if (!sizerNode) break;
            menuWidth = getPropertySize(sizerNode, 'width');
            menuHeight = getPropertySize(sizerNode, 'height');
            sizerNode = sizerNode.firstChild;
        } while (menuWidth < 1 || menuHeight < 1);
    }

    // If we couldn't figure out the actual size, assume size of 220x220
    if (!menuWidth) menuWidth = 220;
    if (!menuHeight) menuHeight = 220;

    // A small buffer that's applied to menu position _in rescue scenarios_, i.e. it's not included by default.
    var buffer = 5;

    var x = void 0,
        y = void 0;
    if (isMobileDevice()) {
        // On mobile devices, centre the menu on the tap
        var halfWidth = menuWidth / 2;
        var halfHeight = menuHeight / 2;
        x = clickX - halfWidth;
        y = clickY - halfHeight;
    } else {
        // On desktop, mimic native context menu placement
        var _right = windowWidth - clickX > menuWidth;
        var left = !_right;
        var top = windowHeight - clickY > menuHeight;
        var _bottom = !top;

        if (_right) x = clickX;
        if (left) x = clickX - menuWidth;
        if (top) y = clickY;
        if (_bottom) y = clickY - menuHeight;
    }

    var right = windowWidth - x - menuWidth;
    var bottom = windowHeight - y - menuHeight;

    if (right < 0) x += right - buffer;
    if (bottom < 0) y += bottom - buffer;

    if (x < 0) x = buffer;
    if (y < 0) y = buffer;

    return { x: x, y: y };
}